import { Injectable } from '@angular/core';
import { HttpClient, HttpParams, HttpHeaders } from '@angular/common/http';
import { Observable } from 'rxjs';
import { SearchResultDTO } from 'app/shared/model/search/search-result-dto.model';

import { SERVER_API_URL } from 'app/app.constants';
import { LikesService } from 'app/entities/likes/likes.service';
import { Exercise } from 'app/shared/model/exercise.model';
import { SearchService } from 'app/search/service/search-service';
import { Statistics } from 'app/shared/model/statistics.model';
import { ArtemisExerciseInfo } from 'app/shared/model/artemis-exercise-info.model';

@Injectable({ providedIn: 'root' })
export class ExerciseService {
  public resourceUrl = SERVER_API_URL + 'api/exerciseFile/';
  public exerciseUrl = SERVER_API_URL + 'api/exercise/';

  constructor(private http: HttpClient, private likesService: LikesService, private searchService: SearchService) {}

  public loadExerciseFile(exerciseId: string, filePath: string): Observable<string> {
    const headers = new HttpHeaders(); // .set('Content-Type', 'text/plain; charset=utf-8');
    const options: HttpParams = new HttpParams();
    options.append('filePath', filePath);

    return this.http.get<string>(this.resourceUrl + exerciseId, {
      headers,
      params: new HttpParams().set('filePath', filePath),
      responseType: 'text' as 'json',
    });
  }

  public loadExercise(exerciseId: string): Observable<SearchResultDTO> {
    return this.http.get<SearchResultDTO>(this.exerciseUrl + exerciseId);
  }

  populateExerciseWithData(exercise: Exercise): Exercise {
    if (exercise !== undefined) {
      exercise.views = 0;
      exercise.downloads = 0;

      this.likesService.getLikesForexerciseID(exercise.originalResult.exerciseId).subscribe(
        (data: number) => {
          exercise.numberOfLikes = data;
        },
        () => alert('Could not load number of likes')
      );

      this.likesService.hasLiked(exercise.originalResult.exerciseId).subscribe(
        (data: boolean) => {
          exercise.userHasLiked = data;
        },
        () => alert('Could not load if user has liked or not')
      );

      this.searchService.getStatisticsForExercise(exercise.originalResult.exerciseId).subscribe(
        (data: Statistics) => {
          exercise.views = data.views!;
          exercise.downloads = data.downloads!;
        },
        () => alert('Could not load exercise statistics')
      );
    }
    return exercise;
  }
  
  /**
   * Used on init of import page, this function passes the retrieved (from Artemis)
   * exerciseUrl to the server
   * @param exerciseUrl
   */
  public importExerciseInfoFromArtemis(exerciseToken: string): Observable<ArtemisExerciseInfo> {
    return this.http.get<ArtemisExerciseInfo>(`${this.exerciseUrl}imported-exercise-info/${exerciseToken}`);
  }

  /**
   * Submits an ArtemisExerciseInfo object to server and triggers its import
   * @param exerciseInfo to submit and import
   */
  public submitExerciseInfoForImport(exerciseInfo: ArtemisExerciseInfo, exerciseToken: string, gitlabGroupId: number) {
    return this.http.post<ArtemisExerciseInfo>(`${this.exerciseUrl}import-exercise/${exerciseToken}/${gitlabGroupId}`, exerciseInfo);
  }

}
