package at.ac.uibk.gitsearch.repository.gitlab;

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Base64;
import java.util.List;
import java.util.Optional;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.gitlab4j.api.Constants.Encoding;
import org.gitlab4j.api.Constants.SearchScope;
import org.gitlab4j.api.Constants.TokenType;
import org.gitlab4j.api.GitLabApi;
import org.gitlab4j.api.GitLabApiException;
import org.gitlab4j.api.models.Namespace;
import org.gitlab4j.api.models.Project;
import org.gitlab4j.api.models.RepositoryFile;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;

//@SpringBootTest(classes = GitsearchApp.class)
public class TestMetaDataGenerator {

	private static final Logger logger = LogManager.getLogger(TestMetaDataGenerator.class);
	
	@Autowired
	GitLabRepository gitLabRepository;
	@Test()
	@Disabled // should only be activated for generation of test data
	public void generateMetadata() throws GitLabApiException, IOException, URISyntaxException {
		GitLabApi gitLabApi = new GitLabApi("https://sharing.codeability-austria.uibk.ac.at/", TokenType.ACCESS, "Hjh1Gk4yFHZs3z7XKonB");

		generateTestData(gitLabApi, null, "Hjh1Gk4yFHZs3z7XKonB");
	}
	
	public byte[] getTestMetaData(File f, int count) throws IOException {
		byte[] fileContent = new byte[] {};
	
		fileContent = Files.readAllBytes(f.toPath());
		if(f.getName().endsWith(".yaml")) {
			String content = new String(fileContent, "UTF-8");
			return content.replace("${title}", "Meta Test Data " + count).replace("${keyword}", "testing" + count).getBytes();
		} else {
			return fileContent;
		}
	}

	/**
	 * just a weird location to generate Test Data :-(. Should be purged as soon as
	 * possible .
	 * 
	 * @throws GitLabApiException
	 * @throws IOException 
	 * @throws URISyntaxException 
	 */
	public void generateTestData(GitLabApi gitLabApi, String issuerURI, String idToken) throws GitLabApiException, IOException, URISyntaxException {
		final String projectPrefix = "apiTest";
		gitLabApi.enableRequestResponseLogging();
		boolean skipDelete = false;
		if (!skipDelete) {
			@SuppressWarnings("unchecked")
			final List<Project> projects = (List<Project>) gitLabApi.getSearchApi().globalSearch(SearchScope.PROJECTS,
					projectPrefix);
			projects.stream().forEach(p -> {
				try {
					gitLabApi.getProjectApi().deleteProject(p.getId());
				} catch (GitLabApiException e) {
					logger.warn("Cannot delete project {}", p.getName(), e);
				}
			});
		}

		final List<Namespace> namespaces = gitLabApi.getNamespaceApi().getNamespaces();
		Optional<Namespace> heathCheckNSO = namespaces.stream().filter(ns -> ns.getName().equals("health-check-tests"))
				.findFirst();
		if (heathCheckNSO.isEmpty()) {
			logger.warn("Namespace health-check-tests not found");
			return;
		}
		Namespace healthNS = heathCheckNSO.get();
		
		final URL resourceFolder = this.getClass().getResource("testProject1");
		final Path dirPath = Paths.get(resourceFolder.toURI());
		File folderDir = dirPath.toFile();
		
		if(!folderDir.exists() || !folderDir.isDirectory()) {
			logger.error("No content folder found!");
			return;
		}
		
		for (int i = 1; i <= 20; i++) {
			Project p = new Project();
			p.setName("apiTest_" + i + "_" + System.currentTimeMillis());
			p.setDescription("Automatically generated testapi");
			p.setNamespace(healthNS);
			p.setSharedWithGroups(null);
			
		
			logger.info("Creating project {} in {}", p.getName(), healthNS.getFullPath());

			Project createdProject = gitLabApi.getProjectApi().createProject(p);

			for(String path: folderDir.list()) {
				File contentFile = new File(folderDir, path);
				RepositoryFile rf = new RepositoryFile();
				rf.setFileName(path);
				String base64encoded = Base64.getEncoder().encodeToString(getTestMetaData(contentFile, i));
				rf.setContent(base64encoded);
				rf.setEncoding(Encoding.BASE64);
				rf.setFilePath(path);
	
				gitLabApi.getRepositoryFileApi().createFile(createdProject.getId(), rf, "master",
						"automagically generated file: "  + path);
			}
		}

	}

}
